package com.gree.mall.contest.controller.mini.user;

import cn.hutool.core.collection.CollectionUtil;
import com.aliyuncs.utils.StringUtils;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.annotation.ApiNotAuth;
import com.gree.mall.contest.bean.user.MyProfitBean;
import com.gree.mall.contest.bean.user.RankBean;
import com.gree.mall.contest.bean.user.RankItemBean;
import com.gree.mall.contest.bean.user.UserWxBean;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.user.UserLogic;
import com.gree.mall.contest.plus.entity.OrderShare;
import com.gree.mall.contest.plus.entity.User;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;

@Slf4j
@RestController
@Tag(name = "小程序用户", description = "小程序用户")
@RequestMapping(value = "/mini/user", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class MiniUserController {


    private final UserLogic userLogic;

    @PostMapping("/bind")
    @Operation(summary = "绑定业务员")
    public ResponseHelper grantMobile(
            @Parameter(description = "用户id", required = true) @RequestParam String userId,
            @Parameter(description = "业务员用户id", required = true) @RequestParam String serviceId
    ) throws Exception {
        userLogic.bind(userId, serviceId);
        return ResponseHelper.success();
    }

    @GetMapping("/qrcode/bind")
    @Operation(summary = "获取业务员绑定小程序码")
    public ResponseHelper bindQrCode(HttpServletRequest request) throws Exception {
        return ResponseHelper.success(userLogic.bindQrCode(request));
    }

    @GetMapping("/rank")
    @Operation(summary = "上月业绩排行榜")
    public ResponseHelper rank(HttpServletRequest request) throws Exception {
        RankBean rank = userLogic.rank(request);
        if (rank != null && CollectionUtil.isNotEmpty(rank.getRankList())) {
            for (RankItemBean bean : rank.getRankList()) {
                if (StringUtils.isNotEmpty(bean.getWorkUserName())) {
                    bean.setWorkUserName(bean.getWorkUserName().charAt(0) + "**");
                }
            }
        }
        return ResponseHelper.success(rank);
    }


    @ApiNotAuth
    @PostMapping("/auth")
    @Operation(summary = "微信授权")
    public ResponseHelper<UserWxBean> auth(
            @Parameter(description = "业务员用户id", required = false) @RequestParam(required = false) String serviceId,
            @Parameter(description = "code", required = true) @RequestParam String code,
            HttpServletRequest request
    ) throws Exception {
        UserWxBean userWxBean;
        log.info("微信授权 code:{} serviceId:{} ", code, serviceId);
        userWxBean = userLogic.addUser(code, serviceId, request);
        userLogic.addVisit(request, userWxBean.getUserId());
        log.info(userWxBean.toString());
        return ResponseHelper.success(userWxBean);
    }

    @ApiNotAuth
    @PostMapping("/mobile/grant")
    @Operation(summary = "授权手机号")
    public ResponseHelper<UserWxBean> grantMobile(
            @Parameter(description = "code", required = true) @RequestParam String code,
            @Parameter(description = "用户id", required = true) @RequestParam String userId,
            HttpServletRequest request
    ) throws Exception {
        UserWxBean wxMobile = userLogic.getWxMobile(code, userId, request);
        return ResponseHelper.success(wxMobile);
    }

    @GetMapping("/user/detail")
    @Operation(summary = "个人信息")
    public ResponseHelper<UserWxBean> detail(
            @Parameter(description = "code", required = true) @RequestParam(required = true) String userId,
            HttpServletRequest request
    ) throws RemoteServiceException {
        UserWxBean userWxBean = userLogic.userDetail(userId, request);
        return ResponseHelper.success(userWxBean);
    }

    @PostMapping("/userinfo/save")
    @Operation(summary = "保存微信账号的用户信息")
    public ResponseHelper<User> saveUserInfo(
            @Parameter(description = "用户id", required = true) @RequestParam String userId,
            @Parameter(description = "用户昵称") @RequestParam(required = false) String nickName,
            @Parameter(description = "性别( 未知 男 女)") @RequestParam(required = false) String sex,
            @Parameter(description = "头像") @RequestParam(required = false) String avatarUrl,
            @Parameter(description = "国家") @RequestParam(required = false) String country,
            @Parameter(description = "省") @RequestParam(required = false) String province,
            @Parameter(description = "城市") @RequestParam(required = false) String city
    ) throws Exception {
        User user = userLogic.saveUserInfo(userId, nickName, sex, avatarUrl, country, province, city);
        return ResponseHelper.success(user);
    }


    @GetMapping("/profit")
    @Operation(summary = "我的收益统计")
    public ResponseHelper<MyProfitBean> userProfit(
            @Parameter(description = "用户id", required = true) @RequestParam(required = true) String userId
    ) throws RemoteServiceException {
        MyProfitBean myProfitBean = userLogic.myProfit(userId);
        return ResponseHelper.success(myProfitBean);
    }

    @GetMapping("/profit/list")
    @Operation(summary = "我的收益明细列表")
    public ResponseHelper<IPage<OrderShare>> userProfitList(
            @Parameter(description = "用户id", required = true) @RequestParam String userId,
            @Parameter(description = "ING=结算中  OVER=已结算  CANCEL=已取消  EXCEPTION=异常结算") @RequestParam(required = false) String status,
            @Parameter(description = "页号", required = true) @RequestParam Integer pageNum,
            @Parameter(description = "页大小", required = true) @RequestParam Integer pageSize
    ) throws RemoteServiceException {
        IPage<OrderShare> myProfit = userLogic.myProfitList(userId, status, pageNum, pageSize);
        return ResponseHelper.success(myProfit);
    }


    @GetMapping("/profit/total")
    @Operation(summary = "总计收益")
    public ResponseHelper<BigDecimal> totalProfit(
            @Parameter(description = "用户id", required = true) @RequestParam(required = true) String userId
    ) throws RemoteServiceException {
        BigDecimal bigDecimal = userLogic.totalProfit(userId);
        return ResponseHelper.success(bigDecimal);
    }

    @GetMapping("/customer")
    @Operation(summary = "我的客户列表")
    public ResponseHelper<Page<User>> customerList(
            @Parameter(description = "用户id", required = true) @RequestParam String userId,
            @Parameter(description = "pageNum", required = true) @RequestParam Integer pageNum,
            @Parameter(description = "pageSize", required = true) @RequestParam Integer pageSize
    ) throws RemoteServiceException {
        IPage<User> userIPage = userLogic.customerList(userId, pageNum, pageSize);
        return ResponseHelper.success(userIPage);
    }


    @GetMapping("/parent")
    @Operation(summary = "我的客户经理")
    public ResponseHelper<User> parentList(
            @Parameter(description = "用户id", required = true) @RequestParam String userId
    ) throws RemoteServiceException {
        User user = userLogic.parentUser(userId);
        return ResponseHelper.success(user);
    }

    @Operation(summary = "续期token")
    @PostMapping("/renewal")
    public ResponseHelper<String> renewal(
            @RequestParam String token
    ) throws RemoteServiceException {
        String newToken = userLogic.renewal(token);
        return ResponseHelper.success(newToken);
    }
}
