package com.gree.mall.contest.controller.pc.merchant;

import cn.hutool.core.lang.TypeReference;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.gree.mall.contest.annotation.ZfireList;
import com.gree.mall.contest.bean.merchant.MerchantAddBean;
import com.gree.mall.contest.bean.merchant.MerchantBean;
import com.gree.mall.contest.bean.merchant.MerchantRegionVO;
import com.gree.mall.contest.bean.merchant.MerchantVO;
import com.gree.mall.contest.bean.zfire.ZfireParamBean;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.merchant.MerchantLogic;
import com.gree.mall.contest.utils.zfire.FieldUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@RequiredArgsConstructor
@RestController
@Tag(name = "商家管理API", description = "商家管理API")
@RequestMapping("/pc/service/provider")
public class MerchantController {

    private final MerchantLogic merchantLogic;

    @ZfireList
    @PostMapping("/list")
    @Operation(summary = "列表")
    public ResponseHelper<IPage<MerchantVO>> list(
            @RequestBody ZfireParamBean zfireParamBean
    ) {
        IPage<MerchantVO> page = merchantLogic.list(zfireParamBean);
        return ResponseHelper.success(page, new TypeReference<MerchantVO>() {});
    }

    @PostMapping("/list/export")
    @Operation(summary = "导出")
    public void listExport(
            @RequestBody ZfireParamBean zfireParamBean,
            HttpServletRequest request,
            HttpServletResponse response
    ) throws Exception {
        //2.查询要导出的内容
        IPage<MerchantVO> baseVOIPage = merchantLogic.list(zfireParamBean);
        //3.导出
        FieldUtils.exportData(baseVOIPage.getRecords(), zfireParamBean.getExportFields(), request, response);
    }

    @ZfireList
    @PostMapping("/region/list")
    @Operation(summary = "商家服务区列表")
    public ResponseHelper<IPage<MerchantRegionVO>> list2(
            @RequestBody ZfireParamBean zfireParamBean
    ) {
        IPage<MerchantRegionVO> page = merchantLogic.list2(zfireParamBean);
        return ResponseHelper.success(page, new TypeReference<MerchantRegionVO>() {});
    }

    @PostMapping("/region/list/export")
    @Operation(summary = "导出")
    public void listExport2(
            @RequestBody ZfireParamBean zfireParamBean,
            HttpServletRequest request,
            HttpServletResponse response
    ) throws Exception {
        //2.查询要导出的内容
        IPage<MerchantRegionVO> page = merchantLogic.list2(zfireParamBean);
        //3.导出
        FieldUtils.exportData(page.getRecords(), zfireParamBean.getExportFields(), request, response);
    }


    @PostMapping("/add")
    @Operation(summary = "新增")
    public ResponseHelper<MerchantBean> add(
            @RequestBody MerchantAddBean merchantAddBean
    ) {
        MerchantBean merchantBean = merchantLogic.add(merchantAddBean);
        return ResponseHelper.success(merchantBean);
    }


    @PostMapping("/update")
    @Operation(summary = "修改")
    public ResponseHelper update(
            @RequestBody MerchantAddBean merchantAddBean
    ) {
        merchantLogic.update(merchantAddBean);
        return ResponseHelper.success();
    }


    @PostMapping("/delete")
    @Operation(summary = "删除")
    public ResponseHelper delete(
            @RequestParam String id
    ) {
        merchantLogic.delete(id);
        return ResponseHelper.success();
    }


    @PostMapping("/detail")
    @Operation(summary = "详情")
    public ResponseHelper<MerchantBean> detail(
            @RequestParam String id
    ) {
        MerchantBean detail = merchantLogic.merchantDetail(id);
        return ResponseHelper.success(detail);
    }



}
