package com.gree.mall.contest.logic.admin;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.gree.mall.contest.bean.admin.AdminModuleBean;
import com.gree.mall.contest.bean.admin.AdminModuleVO;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.plus.entity.AdminModule;
import com.gree.mall.contest.plus.service.AdminModuleService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

@Service
@Slf4j
@RequiredArgsConstructor
public class AdminModuleLogic {

    private final AdminModuleService adminModuleService;


    @Transactional
    public void add(AdminModuleBean adminModule){
        Integer level = 1;
        if(!adminModule.getParentId().equals("0")){
            AdminModule parent = adminModuleService.getById(adminModule.getParentId());
            if(parent == null){
                throw new RemoteServiceException("请选择正确的父级菜单");
            }
            level = parent.getLevel() + 1;
            parent.updateById();
        }
        Long count = adminModuleService.lambdaQuery()
                .eq(AdminModule::getCode,adminModule.getCode())
                .eq(AdminModule::getParentId,adminModule.getParentId())
                .count();
        if(count > 0){
            throw new RemoteServiceException("code或者url重复");
        }
        adminModule.setLevel(level);
        adminModule.setCreateTime(new Date());
        adminModule.insert();
        //添加子集
        if(CollectionUtils.isNotEmpty(adminModule.getChildList())){
            for (AdminModule module : adminModule.getChildList()) {
                module.setLevel(adminModule.getLevel() + 1);
                module.setParentId(adminModule.getModuleId());
            }
            adminModuleService.saveBatch(adminModule.getChildList());
        }
    }


    public void update(AdminModule adminModule){
        Integer level = 1;
        if(adminModule.getParentId() != null && !adminModule.getParentId().equals("0")){
            AdminModule parent = adminModuleService.getById(adminModule.getParentId());
            if(parent == null){
                throw new RemoteServiceException("请选择正确的父级菜单");
            }
            level = parent.getLevel() + 1;
        }
        Long count = adminModuleService.lambdaQuery()
                .ne(AdminModule::getModuleId,adminModule.getModuleId())
                .eq(AdminModule::getCode,adminModule.getCode())
                .eq(AdminModule::getParentId,adminModule.getParentId())
                .count();
        if(count > 0){
            throw new RemoteServiceException("code或者url重复");
        }
        if(adminModule.getType() != 3) {
            adminModule.setLevel(level);
        }
        adminModule.updateById();
    }


    public void delete(String id){
        adminModuleService.removeById(id);
    }


    public AdminModuleVO detail(String id){
        AdminModule adminModule = adminModuleService.getById(id);
        AdminModuleVO adminModuleVO = BeanUtil.copyProperties(adminModule, AdminModuleVO.class);
        //查询子集
        if(adminModule.getType() == 2){
            List<AdminModule> list = adminModuleService.lambdaQuery().eq(AdminModule::getParentId, adminModule.getModuleId()).list();
            List<AdminModuleVO> adminModuleVOS = BeanUtil.copyToList(list, AdminModuleVO.class);
            adminModuleVO.setChildList(adminModuleVOS);
        }
        return adminModuleVO;
    }


}
