package com.gree.mall.contest.logic.admin;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.excel.util.StringUtils;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.bean.admin.AdminRoleBean;
import com.gree.mall.contest.bean.admin.AdminUserCom;
import com.gree.mall.contest.enums.admin.RoleTypeEnum;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.plus.entity.AdminRole;
import com.gree.mall.contest.plus.entity.AdminUser;
import com.gree.mall.contest.plus.service.AdminRoleService;
import com.gree.mall.contest.plus.service.AdminUserService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

@Service
@Slf4j
@RequiredArgsConstructor
public class AdminRoleLogic {

    private final AdminRoleService adminRoleService;
    private final AdminUserService adminUserService;
    private final CommonLogic commonLogic;
//    private final ServiceProviderService serviceProviderService;


    /**
     * 角色列表
     */
    public IPage<AdminRoleBean> listPage(HttpServletRequest request, String adminWebsitId, Integer pageNo, Integer pageSize) {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);

        String type = adminRoleService.getById(adminUser.getRoleId()).getType();
        if("admin".equals(adminUser.getUserName())){
            type = RoleTypeEnum.A.getCode();
        }

        Page page = adminRoleService.lambdaQuery()
                .eq(StringUtils.isNotBlank(adminUser.getServiceProviderId()), AdminRole::getServiceProviderId, adminUser.getServiceProviderId())
                .eq(StringUtils.isNotBlank(adminWebsitId),AdminRole::getAdminWebsitId,adminWebsitId)
                .orderByDesc(Arrays.asList(AdminRole::getCreateTime, AdminRole::getType))
                .page(new Page<>(pageNo, pageSize));
        List<AdminRoleBean> adminRoleBeans = BeanUtil.copyToList(page.getRecords(), AdminRoleBean.class);
        for(AdminRoleBean bean : adminRoleBeans){
            Long count = adminUserService.lambdaQuery().eq(AdminUser::getRoleId, bean.getAdminRoleId()).count();
            bean.setAccountNum(count);
        }
        page.setRecords(adminRoleBeans);
        return page;
    }

    /**
     * 新增角色
     */
    public void add(HttpServletRequest request, AdminRole adminRole) {
        AdminUserCom adminUser = commonLogic.getAdminUser();
        AdminRole adminRole1= adminRoleService.getById(adminUser.getRoleId());
        //默认部门角色
//        ServiceProvider serviceProvider = serviceProviderService.getById(adminUser.getServiceProviderId());

        adminRole.setType(StringUtils.isNotBlank(adminUser.getServiceProviderId())?"A":"" == null ? "C":"B");

        adminRole.setServiceProviderId(adminUser.getServiceProviderId());
        adminRole.setAdminWebsitId(adminUser.getAdminWebsitId());
        adminRole.setCreateTime(new Date());
        adminRole.setCreateBy(adminUser.getNickName());
        adminRoleService.save(adminRole);
    }

    /**
     * 修改角色
     */
    @Transactional
    public void update(AdminRole adminRole) {
        adminRoleService.updateById(adminRole);

        if (adminRole.getName() != null) {
            adminUserService.lambdaUpdate()
                    .set(AdminUser::getRoleName, adminRole.getName())
                    .eq(AdminUser::getRoleId, adminRole.getAdminRoleId()).update();
        }
    }

    /**
     * 删除角色
     */
    public void delete(String adminRoleId) throws RemoteServiceException {
        //不允许删除使用该角色的帐号
        Long count = adminUserService.lambdaQuery().eq(AdminUser::getRoleId, adminRoleId).count();
        if (count > 0) {
            throw new RemoteServiceException("共有" + count + "个帐号使用该角色,不可删除");
        }
        if(adminRoleId.length() < 5){
            throw new RemoteServiceException("暂不可删除该角色，请联系相关人员");
        }
        adminRoleService.removeById(adminRoleId);
    }

    /**
     * 详情
     */
    public AdminRole detail(String adminRoleId) {
        return adminRoleService.getById(adminRoleId);
    }


}
