package com.gree.mall.contest.logic.admin;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.bean.admin.AdminUserCom;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.plus.entity.OperationLog;
import com.gree.mall.contest.plus.service.OperationLogService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Slf4j
@RequiredArgsConstructor
public class OperationLogLogic {

    private final OperationLogService operationLogService;

    private final CommonLogic commonLogic;

    /**
     * 操作日志列表分页
     */
    public IPage<OperationLog> listPage(HttpServletRequest request, String moduleName, String companyWechatName, String nickName, String startTime, String endTime, Integer pageNo, Integer pageSize){
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        List<String> companyWechatIds = adminUser.getCompanyWechatIds();

        return operationLogService.lambdaQuery()
                .in(CollectionUtils.isNotEmpty(companyWechatIds),OperationLog::getCompanyWechatId,companyWechatIds)
                .likeRight(StringUtils.isNotBlank(companyWechatName),OperationLog::getCompanyName,companyWechatName)
                .likeRight(StringUtils.isNotBlank(nickName),OperationLog::getNickName,nickName)
                .like(OperationLog::getModuleName,moduleName)
                .between(StringUtils.isNotBlank(startTime) && StringUtils.isNotBlank(endTime),
                        OperationLog::getCreateTime,startTime,endTime)
                .orderByDesc(Arrays.asList(OperationLog::getCreateTime,OperationLog::getOperationLogId))
                .page(new Page<>(pageNo,pageSize));
    }

    /**
     * 所属模块
     */
    public List<String> moduleList(HttpServletRequest request){
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        List<String> companyWechatIds = adminUser.getCompanyWechatIds();

        List<OperationLog> distinctModuleName = operationLogService.lambdaQuery()
                .select(OperationLog::getModuleName)
                .in(CollectionUtils.isNotEmpty(companyWechatIds),OperationLog::getCompanyWechatId,companyWechatIds)
                .groupBy(OperationLog::getModuleName)
                .list();
        if(distinctModuleName.size() == 0){
            return new ArrayList<>();
        }
        List<String> collect = distinctModuleName.stream().map(OperationLog::getModuleName).distinct().collect(Collectors.toList());
        return collect;
    }

    /**
     * 详情
     */
    public OperationLog detail(String operationLogId){
        return operationLogService.getById(operationLogId);
    }


}
