package com.gree.mall.contest.logic.coupon;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.bean.ExcelData;
import com.gree.mall.contest.bean.admin.AdminUserCom;
import com.gree.mall.contest.bean.coupon.*;
import com.gree.mall.contest.bean.user.CurrentCompanyWechat;
import com.gree.mall.contest.bean.user.UserCouponCountBean;
import com.gree.mall.contest.commonmapper.CouponDateMapper;
import com.gree.mall.contest.commonmapper.CoustomUserCouponMapper;
import com.gree.mall.contest.commonmapper.CustomCoupouMapper;
import com.gree.mall.contest.constant.Constant;
import com.gree.mall.contest.enums.QrCodeEnum;
import com.gree.mall.contest.enums.UserTypeEnum;
import com.gree.mall.contest.enums.coupon.CouponActiveTypeEnum;
import com.gree.mall.contest.enums.coupon.CouponCrowdEnum;
import com.gree.mall.contest.enums.coupon.CouponFlagEnum;
import com.gree.mall.contest.enums.coupon.CouponTypeEnum;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.logic.common.WechatLogic;
import com.gree.mall.contest.logic.user.UserLogic;
import com.gree.mall.contest.plus.entity.*;
import com.gree.mall.contest.plus.service.*;
import com.gree.mall.contest.utils.CommonUtils;
import com.gree.mall.contest.utils.excel.ExcelUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.integration.redis.util.RedisLockRegistry;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.stream.Collectors;

@Service
@Slf4j
@RequiredArgsConstructor
public class CouponLogic {

    private final CouponService couponService;
    private final CouponGoodsService couponGoodsService;
    private final CouponDateMapper couponDateMapper;
    private final UserCouponService userCouponService;
    private final CouponUserService couponUserService;
    private final UserService userService;
    private final CommonLogic commonLogic;
    private final RedisLockRegistry redisLockRegistry;
    private final CustomCoupouMapper customCoupouMapper;
    private final WechatLogic wechatLogic;
    private final CoustomUserCouponMapper coustomUserCouponMapper;

    /**
     * 优惠券列表
     */
    public IPage<CouponPageBean> page(String couponName, String couponType, String flag, Integer pageNum, Integer pageSize, HttpServletRequest request) {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        List<String> companyWechatIds = adminUser.getCompanyWechatIds();

        return couponDateMapper.pageCoupon(new Page<>(pageNum, pageSize), companyWechatIds, couponName, couponType, flag);
    }


    public void export(HttpServletRequest request, HttpServletResponse response, String couponName, String couponType, String flag) throws Exception {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        List<String> companyWechatIds = adminUser.getCompanyWechatIds();

        List<CouponExportBean> couponExportBeans = couponDateMapper.exportCoupon(companyWechatIds, couponName, couponType, flag);
        //List<CouponExportBean> couponSelfBeans =  couponDateMapper.exportSelfCoupon(companyWechatIds, couponName, couponType, flag);
//        if(CollectionUtils.isNotEmpty(couponSelfBeans)){
//            couponExportBeans.addAll(couponSelfBeans);
//        }

        String[] titles = {"优惠券名称", "发放数量", "领取数量", "使用数量", "优惠券金额", "领取来源-网点名称", "领取来源-业务员昵称", "领取来源-业务员电话", "优惠券状态", "领券客户昵称", "领券客户电话",
                "用券时间", "订单号", "订单实付金额"};
        ExcelData excelData = new ExcelData();
        List<List<Object>> rows = new ArrayList<>();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        for (CouponExportBean couponExportBean : couponExportBeans) {
            List<Object> row = new ArrayList<>();
            row.add(couponExportBean.getCouponName());
            row.add(couponExportBean.getCouponAmount());
            row.add(couponExportBean.getReceiveAmount());
            row.add(couponExportBean.getUsedAmount());
            row.add(couponExportBean.getCouponValue());
            row.add(couponExportBean.getWebsitName());
            row.add(couponExportBean.getWorkName());
            row.add(couponExportBean.getWorkPhone());
            row.add(couponExportBean.getStatus() ? "已使用" : "未使用");
            row.add(couponExportBean.getNickName());
            row.add(couponExportBean.getMobile());
            row.add(couponExportBean.getUsedTime() == null ? "" : simpleDateFormat.format(couponExportBean.getUsedTime()));
            row.add(couponExportBean.getOrderId());
            row.add(couponExportBean.getPayAmount() == null ? "" : couponExportBean.getPayAmount().toString());
            rows.add(row);
        }
        String fileName = String.format("优惠券导出_%s.xlsx", simpleDateFormat.format(new Date()));
        excelData.setTitles(Arrays.asList(titles));
        excelData.setName("优惠券");
        excelData.setRows(rows);
        ExcelUtils.exportExcel(request, response, fileName, excelData, -1, null);
    }

    /**
     * 券详情
     *
     * @param couponId
     * @return
     */
    public CouponDetailBean detail(String couponId) {
        Coupon coupon = couponService.getById(couponId);
        CouponDetailBean couponBean = new CouponDetailBean();
        BeanUtils.copyProperties(coupon, couponBean);
        couponBean.setGoodsList(couponGoodsService.lambdaQuery().eq(CouponGoods::getCouponId, couponId).list());
        return couponBean;
    }

    /**
     * 新增券
     *
     * @param couponBean
     */
    @Transactional(rollbackFor = Exception.class)
    public void add(HttpServletRequest request, CouponBean couponBean) throws RemoteServiceException {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);

        if (couponBean.getActiveType() == 1) {
            if (Objects.isNull(couponBean.getActiveStartTime()) || Objects.isNull(couponBean.getActiveEndTime())) {
                throw new RemoteServiceException("使用时间区间不能为空！");
            }
            if (couponBean.getActiveEndTime().compareTo(couponBean.getActiveStartTime()) < 0) {
                throw new RemoteServiceException("使用结束时间不能少于开始时间！");
            }
        }
        couponBean.setFlag(CouponFlagEnum.WAIT.toString());
        couponBean.setLeftAmount(couponBean.getCouponAmount());
        couponBean.setCreateTime(new Date());
        couponBean.setCompanyWechatId(adminUser.getLoginCompanyWechatId());
        couponBean.setCompanyName(adminUser.getLoginCompanyName());
        couponBean.setActiveType(couponBean.getActiveDay() != null ? 2 : 1);
        couponService.save(couponBean);
        String couponId = couponBean.getCouponId();
        if (couponBean.getCouponType().trim().equals(CouponTypeEnum.GOODS.getName()) && couponBean.getGoodsList() != null) {
            List<CouponGoods> couponGoodsList = new ArrayList<>();
            for (CouponGoods couponGoods : couponBean.getGoodsList()) {
                couponGoods.setCouponId(couponId);
                couponGoods.setCreateTime(new Date());
                couponGoods.setCompanyWechatId(adminUser.getLoginCompanyWechatId());
                couponGoods.setCompanyName(adminUser.getLoginCompanyName());
                couponGoodsList.add(couponGoods);
            }
            couponGoodsService.saveBatch(couponGoodsList);
        }

        //指定会员
        if (couponBean.getReceiveCrowd().equals(CouponCrowdEnum.SPECIFY.getFlag()) && couponBean.getUserList() != null) {
            List<CouponUser> couponUserList = new ArrayList<>();
//            List<UserCoupon> userCouponList = new ArrayList<>();
            for (CouponUserTypeBean couponUserTypeBean : couponBean.getUserList()) {
                CouponUser couponUser = new CouponUser();
                couponUser.setCouponId(couponId);
                couponUser.setUserId(couponUserTypeBean.getUserId());
                couponUser.setUserName(couponUserTypeBean.getUserName());
                couponUser.setCreateTime(new Date());
                couponUser.setCompanyName(adminUser.getLoginCompanyName());
                couponUser.setCompanyWechatId(adminUser.getLoginCompanyWechatId());
                couponUserList.add(couponUser);
//                if (couponUserTypeBean.getType().equals(UserTypeEnum.SERVICE.toString())) {
//                    userCouponList.add(assignCoupon(adminUser, couponBean, couponUserTypeBean.getUserId()));
//                }

            }

            if (couponUserList.size() > 0) {
                couponUserService.saveBatch(couponUserList);
            }

//            if (userCouponList.size() > 0) {
//                userCouponService.saveBatch(userCouponList);
//            }

        }
        //全部业务员
        if (couponBean.getReceiveCrowd() == CouponCrowdEnum.ALL_SERVICE.getFlag()) {
            List<UserCoupon> userCouponList = new ArrayList<>();
            List<User> userList = userService.lambdaQuery()
                    .eq(User::getType, UserTypeEnum.SERVICE.toString())
                    .eq(User::getCompanyWechatId, adminUser.getLoginCompanyWechatId())
                    .list();
            List<CouponUser> couponUserList = new ArrayList<>();
            for (User user : userList) {
                userCouponList.add(assignCoupon(adminUser, couponBean, user.getUserId()));
                CouponUser couponUser = new CouponUser();
                couponUser.setCouponId(couponId);
                couponUser.setUserId(user.getUserId());
                couponUser.setUserName(user.getNickName());
                couponUser.setCreateTime(new Date());
                couponUser.setCompanyWechatId(adminUser.getLoginCompanyWechatId());
                couponUser.setCompanyName(adminUser.getLoginCompanyName());
                couponUserList.add(couponUser);
            }
            if (userCouponList.size() > 0) {
                userCouponService.saveBatch(userCouponList);
            }
            if (couponUserList.size() > 0) {
                couponUserService.saveBatch(couponUserList);
            }
        }
        //全部普通用户
        if (couponBean.getReceiveCrowd().equals(CouponCrowdEnum.ALL_GENRERAL.getFlag())) {
            List<CouponUser> couponUserList = new ArrayList<>();
            List<User> userList = userService.lambdaQuery()
                    .eq(User::getType, UserTypeEnum.GENERAL.toString())
                    .eq(User::getCompanyWechatId, adminUser.getLoginCompanyWechatId())
                    .list();
            for (User user : userList) {
                CouponUser couponUser = new CouponUser();
                couponUser.setCouponId(couponId);
                couponUser.setUserId(user.getUserId());
                couponUser.setUserName(user.getNickName());
                couponUser.setCreateTime(new Date());
                couponUser.setCompanyWechatId(adminUser.getLoginCompanyWechatId());
                couponUser.setCompanyName(adminUser.getLoginCompanyName());
                couponUserList.add(couponUser);
            }
            if (couponUserList.size() > 0) {
                couponUserService.saveBatch(couponUserList);
            }
        }


    }


    private UserCoupon assignCoupon(AdminUserCom adminUserCom, CouponBean couponBean, String userId) {
        UserCoupon userCoupon = new UserCoupon();
        userCoupon.setCompanyWechatId(adminUserCom.getLoginCompanyWechatId());
        userCoupon.setCouponName(adminUserCom.getLoginCompanyName());
        userCoupon.setCouponId(couponBean.getCouponId());
        userCoupon.setUserId(userId);
        userCoupon.setReceiveTime(new Date());
        userCoupon.setCouponName(couponBean.getCouponName());
        userCoupon.setCouponType(couponBean.getCouponType());
        userCoupon.setOrderAmount(couponBean.getOrderAmount());
        userCoupon.setCouponValue(couponBean.getCouponValue());
        userCoupon.setStatus(false);
        //userCoupon.setTransferType(true);
        userCoupon.setLeftShareTimes(couponBean.getShareTimes());
        if (couponBean.getActiveDay() == null) {
            couponBean.setActiveDay(0);
        }
        if (couponBean.getActiveType().equals(CouponActiveTypeEnum.COMMON.getActiveType())) {
            userCoupon.setActiveStartTime(couponBean.getActiveStartTime());
            userCoupon.setActiveEndTime(couponBean.getActiveEndTime());
        }
        //当日起多少天
        if (couponBean.getActiveType().equals(CouponActiveTypeEnum.TODAY.getActiveType())) {
            userCoupon.setActiveStartTime(DateUtil.parseTime(DateUtil.formatDate(DateUtil.date()) + " 00:00:00"));
            userCoupon.setActiveEndTime(DateUtil.parseTime(DateUtil.formatDate(DateUtil.offsetDay(DateUtil.date(), couponBean.getActiveDay())) + " 23:59:59"));
        }
        //次日起多少天
        if (couponBean.getActiveType().equals(CouponActiveTypeEnum.NEXTDAY.getActiveType())) {
            userCoupon.setActiveStartTime(DateUtil.parseTime(DateUtil.formatDate(DateUtil.offsetDay(DateUtil.date(), 1)) + " 00:00:00"));
            userCoupon.setActiveEndTime(DateUtil.parseTime(DateUtil.formatDate(DateUtil.offsetDay(DateUtil.date(), couponBean.getActiveDay() + 1)) + " 23:59:59"));
        }
        userCoupon.setState(new Date().before(couponBean.getActiveEndTime()) && new Date().after(couponBean.getActiveStartTime()));
        return userCoupon;
    }

    public void reissueCoupon(HttpServletRequest request, ReissueCouponBean reissueCouponBean) throws RemoteServiceException {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        String loginCompanyWechatId = adminUser.getLoginCompanyWechatId();
        String loginCompanyName = adminUser.getLoginCompanyName();

        Coupon coupon = couponService.getById(reissueCouponBean.getCouponId());
        if (coupon == null) {
            throw new RemoteServiceException("优惠券不存在");
        }
        if (coupon.getTransferType() != null && !coupon.getTransferType()) {
            throw new RemoteServiceException("优惠券不可以补发");
        }
        if (coupon.getObtainEndTime().compareTo(new Date()) <= 0) {
            throw new RemoteServiceException("优惠券超过领取时间");
        }
        List<CouponUserTypeBean> userIds = reissueCouponBean.getUserIds();
        if (CollectionUtil.isEmpty(userIds)) {
            userIds = couponDateMapper.reissueCouponUser(loginCompanyWechatId, coupon.getCouponId());
        }
        List<CouponUser> couponUserList = new ArrayList<>();
        List<UserCoupon> userCouponList = new ArrayList<>();
        for (CouponUserTypeBean couponUserTypeBean : userIds) {
            CouponUser couponUser = new CouponUser();
            couponUser.setCouponId(coupon.getCouponId());
            couponUser.setUserId(couponUserTypeBean.getUserId());
            couponUser.setUserName(couponUserTypeBean.getUserName());
            couponUser.setCreateTime(new Date());
            couponUser.setCompanyWechatId(loginCompanyWechatId);
            couponUser.setCompanyName(loginCompanyName);
            couponUserList.add(couponUser);
            CouponBean couponBean = new CouponBean();
            BeanUtils.copyProperties(coupon, couponBean);
            userCouponList.add(assignCoupon(adminUser, couponBean, couponUserTypeBean.getUserId()));
        }

        if (couponUserList.size() > 0) {
            couponUserService.saveBatch(couponUserList);
        }

        if (userCouponList.size() > 0) {
            userCouponService.saveBatch(userCouponList);
        }

    }


    /**
     * 修改券
     *
     * @param couponBean
     */
    @Transactional(rollbackFor = Exception.class)
    public void update(HttpServletRequest request, CouponBean couponBean) throws RemoteServiceException {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        String loginCompanyName = adminUser.getLoginCompanyName();
        String loginCompanyWechatId = adminUser.getLoginCompanyWechatId();

        Coupon coupon = couponService.getById(couponBean.getCouponId());
        couponBean.setCreateTime(null);
        Date now = new Date();
        if (coupon.getFlag().trim().equals(CouponFlagEnum.WAIT.toString())) {
            //待发布处理
            couponService.updateById(couponBean);
            //清除商品
            if (couponBean.getGoodsList() != null && couponBean.getGoodsList().size() > 0) {
                couponGoodsService.lambdaUpdate()
                        .eq(CouponGoods::getCouponId, couponBean.getCouponId()).remove();
                for (CouponGoods couponGoods : couponBean.getGoodsList()) {
                    couponGoods.setCouponId(couponBean.getCouponId());
                    couponGoods.setCreateTime(now);
                    couponGoods.setCompanyWechatId(loginCompanyWechatId);
                    couponGoods.setCompanyName(loginCompanyName);
                    couponGoodsService.save(couponGoods);
                }
            }

            if (couponBean.getUserList() != null && couponBean.getUserList().size() > 0) {
                couponUserService.lambdaUpdate()
                        .eq(CouponUser::getCouponId, couponBean.getCouponId())
                        .remove();
                List<CouponUser> couponUserList = new ArrayList<>();
                for (CouponUserTypeBean couponUserTypeBean : couponBean.getUserList()) {
                    CouponUser couponUser = new CouponUser();
                    couponUser.setCouponId(couponBean.getCouponId());
                    couponUser.setUserId(couponUserTypeBean.getUserId());
                    couponUser.setUserName(couponUserTypeBean.getUserName());
                    couponUser.setCreateTime(new Date());
                    couponUser.setCompanyWechatId(loginCompanyWechatId);
                    couponUser.setCompanyName(loginCompanyName);
                    couponUserList.add(couponUser);
                    if (couponUserTypeBean.getType().equals(UserTypeEnum.SERVICE.toString())) {
                        UserCoupon userCoupon = userCouponService.lambdaQuery()
                                .eq(UserCoupon::getCouponId, couponBean.getCouponId())
                                .eq(UserCoupon::getUserId, couponUserTypeBean.getUserId())
                                //.eq(UserCoupon::getTransferType, true)
                                .one();
                        if (userCoupon == null) {
                            userCouponService.save(assignCoupon(adminUser, couponBean, couponUserTypeBean.getUserId()));
                        }

                    }
                }
                if (couponUserList.size() > 0) {
                    couponUserService.saveBatch(couponUserList);
                }
            }
        } else if (!coupon.getFlag().trim().equals(CouponFlagEnum.CANCEL.toString()) && coupon.getDisplayTime().before(now) && coupon.getObtainEndTime().after(now)) {
            //进行中处理
            if (coupon.getCouponAmount() > couponBean.getCouponAmount()) {
                throw new RemoteServiceException("修改优惠券发放总量时只能增加不能减少！");
            }
            couponService.lambdaUpdate()
                    .eq(Coupon::getCouponId, couponBean.getCouponId())
                    .set(StringUtils.isNotBlank(couponBean.getCouponName()), Coupon::getCouponName, couponBean.getCouponName())
                    .set(couponBean.getCouponAmount() > coupon.getCouponAmount(), Coupon::getCouponAmount, couponBean.getCouponAmount())
                    .set(!Objects.isNull(couponBean.getSortNum()), Coupon::getSortNum, couponBean.getSortNum())
                    .set(StringUtils.isNotBlank(couponBean.getNote()), Coupon::getNote, couponBean.getNote())
                    .set(StringUtils.isNotBlank(couponBean.getRemark()), Coupon::getRemark, couponBean.getRemark())
                    .update();
            //更改已领取的券名称
            if (StringUtils.isNotBlank(couponBean.getCouponName())) {
                userCouponService.lambdaUpdate()
                        .eq(UserCoupon::getCouponId, couponBean.getCouponId())
                        .set(UserCoupon::getCouponName, couponBean.getCouponName())
                        .update();
            }
        }
    }

    /**
     * 券标志
     *
     * @param couponId
     * @param startDate
     * @param endDate
     * @param flag
     */
    public void updateFlag(String couponId, String startDate, String endDate, String flag) throws RemoteServiceException {
        if (flag.trim().equals(CouponFlagEnum.START.toString())) {
            if (StringUtils.isBlank(startDate) || StringUtils.isBlank(endDate)) {
                throw new RemoteServiceException("时间不能为空！");
            }
            if (endDate.compareTo(startDate) < 0) {
                throw new RemoteServiceException("结束时间不能少于开始时间！");
            }

            couponService.lambdaUpdate()
                    .eq(Coupon::getCouponId, couponId)
                    .set(Coupon::getReleaseStartTime, startDate)
                    .set(Coupon::getReleaseEndTime, endDate)
                    .set(Coupon::getFlag, CouponFlagEnum.START.toString())
                    .update();
        } else {
            couponService.lambdaUpdate()
                    .eq(Coupon::getCouponId, couponId)
                    .set(Coupon::getFlag, CouponFlagEnum.CANCEL.toString())
                    .update();

            userCouponService.lambdaUpdate()
                    .set(UserCoupon::getState, false)
                    .set(UserCoupon::getLeftShareTimes, 0)
                    .set(UserCoupon::getActiveEndTime, DateUtil.formatDate(DateUtil.date()) + " 00:00:00")
                    .eq(UserCoupon::getCouponId, couponId)
                    .eq(UserCoupon::getStatus, false)
                    .update();
        }
    }

    /**
     * 券数据
     *
     * @param couponId
     * @return
     */
    public CouponDataBean data(String couponId) {
        return couponDateMapper.data(couponId);
    }


    /**
     * 修改优惠券是否可以转让
     */
    public void updateUserCouponTransfer(List<String> userIds, Boolean transferType) {
        userCouponService.lambdaUpdate()
                .set(UserCoupon::getTransferType, transferType)
                .in(UserCoupon::getUserId, userIds)
                .update();
    }

    /**
     * 我的优惠券
     */
    public IPage<UserCoupon> userCouponList(String userId, Integer pageNo, Integer pageSize) {
        return userCouponService.lambdaQuery()
                .eq(UserCoupon::getUserId, userId)
                .orderByDesc(UserCoupon::getReceiveTime)
                .page(new Page<>(pageNo, pageSize));
    }

    /**
     * 领取优惠券明细
     *
     * @param userCouponId
     * @param pageNo
     * @param pageSize
     * @return
     */
    public IPage<UserCouponBean> receiveCouponDetail(String userCouponId, Integer pageNo, Integer pageSize) {

        IPage page = userCouponService.lambdaQuery()
                .eq(UserCoupon::getTransferor, userCouponId)
                .eq(UserCoupon::getTransferType, 0)
                .orderByDesc(UserCoupon::getReceiveTime)
                .page(new Page<>(pageNo, pageSize));

        List<UserCouponBean> list = new ArrayList<>();
        for (Object o : page.getRecords()) {
            UserCoupon userCoupon = (UserCoupon) o;
            UserCouponBean couponUser = new UserCouponBean();
            BeanUtils.copyProperties(userCoupon, couponUser);
            User user = userService.getById(userCoupon.getUserId());
            couponUser.setUserName(user.getNickName());
            couponUser.setMobile(user.getMobile());
            list.add(couponUser);
        }
        page.setRecords(list);
        return page;

    }

    public IPage<User> userList(HttpServletRequest request,
                                String tel, Date registerTime, String keyword, String type, Integer pageNo, Integer pageSize) {

        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        List<String> companyWechatIds = adminUser.getCompanyWechatIds();

        return userService.lambdaQuery()
                .in(CollectionUtil.isNotEmpty(companyWechatIds), User::getCompanyWechatId, companyWechatIds)
                .eq(StringUtils.isNotEmpty(type), User::getType, type)
                .ge(registerTime != null, User::getCreateTime, registerTime)
                .like(StringUtils.isNotEmpty(tel), User::getMobile, tel.trim())
                .like(StringUtils.isNotEmpty(keyword), User::getNickName, keyword.trim())
                .orderByAsc(User::getCreateTime)
                .page(new Page<>(pageNo, pageSize));
    }

    /**
     * 订单取消,恢复优惠券
     */
    @Transactional
    public void refundCoupon(String userCouponId) {
        if (StringUtils.isEmpty(userCouponId)) {
            return;
        }
        userCouponService.lambdaUpdate()
                .set(UserCoupon::getStatus, false)
                .set(UserCoupon::getUsedTime, null)
                .eq(UserCoupon::getId, userCouponId)
                .update();

        if (StringUtils.isEmpty(userCouponId)) {
            return;
        }
        log.info("订单取消,恢复用户优惠券id{}", userCouponId);
        UserCoupon userCoupon = userCouponService.getById(userCouponId);
        if (userCoupon.getTransferType() != null && userCoupon.getTransferType()) {
            userCoupon.setLeftShareTimes(userCoupon.getLeftShareTimes() + 1);
            userCoupon.updateById();
        }
    }

    /**
     * 优惠券过期提醒
     */
    public Boolean timeout(String userId) {
        Date timeByDay = DateUtil.offsetDay(new Date(), -3);
        Long count = userCouponService.lambdaQuery().eq(UserCoupon::getUserId, userId)
                .between(UserCoupon::getActiveEndTime, timeByDay, new Date()).count();
        return count > 0;
    }

    /**
     * 统计 未使用 ，已使用，已过期优惠券
     *
     * @param userId
     * @return
     */
    public UserCouponCountBean count(String userId) {
        UserCouponCountBean userCouponCountBean = new UserCouponCountBean();
        List<UserCoupon> list = userCouponService.lambdaQuery().eq(UserCoupon::getUserId, userId).list();
        if (list.size() == 0) {
            return userCouponCountBean;
        }
        //未使用数量,已使用数量，已过期数量
        int wsy = 0, ysy = 0, ygq = 0;
        for (UserCoupon userCoupon : list) {

            if (userCoupon.getStatus()) {
                ysy++;
            } else {
                if (userCoupon.getActiveEndTime().getTime() > new Date().getTime()) {
                    wsy++;
                } else {
                    ygq++;
                }
            }
        }
        userCouponCountBean.setWsy(wsy);
        userCouponCountBean.setYsy(ysy);
        userCouponCountBean.setYgq(ygq);
        return userCouponCountBean;
    }

    /**
     * 我的优惠券
     *
     * @param userId
     * @param pageNum
     * @param pageSize
     * @return
     */
    public IPage<UserCoupon> miniPage(String userId, Integer type, Integer pageNum, Integer pageSize) {

        Date date = new Date();

        LambdaQueryWrapper<UserCoupon> qw = new LambdaQueryWrapper<>();
        qw.eq(UserCoupon::getUserId, userId);

        //未使用
        if (type != null && type == 0) {
            qw.eq(UserCoupon::getStatus, type).ge(UserCoupon::getActiveEndTime, date);
        }
        //已使用
        if (type != null && type == 1) {
            qw.eq(UserCoupon::getStatus, type);
        }
        //已过期
        if (type != null && type == 2) {
            qw.eq(UserCoupon::getStatus, 0);
            qw.lt(UserCoupon::getActiveEndTime, date);
        }
        qw.orderByDesc(UserCoupon::getReceiveTime);
        IPage<UserCoupon> page = userCouponService.page(new Page<>(pageNum, pageSize), qw);
        return page;
    }

    public List<UserCoupon> expiring(String userId) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(new Date());
        cal.add(Calendar.DATE, 1);
        return userCouponService.lambdaQuery()
                .eq(UserCoupon::getUserId, userId)
                .le(UserCoupon::getActiveEndTime, cal.getTime())
                .list();
    }

    public List<CouponObtainBean> allObtainCoupou(String userId, HttpServletRequest request) {
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);
        List<CouponObtainBean> couponObtainBeans = customCoupouMapper.listObtainCoupou(userId, currentCompanyWechat.getCompanyWechatId());
        //发券
        try {
            List<CouponObtainBean> obtions = couponObtainBeans.stream().filter(v -> v.getObtainType() == 1).collect(Collectors.toList());
            if (obtions.size() > 0) {
                List<String> couponIds = obtions.stream().map(CouponObtainBean::getCouponId).collect(Collectors.toList());
                for (String couponId : couponIds) {
                    Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.LOCK_COUPON + couponId);
                    if (!obtain.tryLock(10, TimeUnit.SECONDS)) {
                        log.error("系统繁忙，领券失败,userId:{},couponId:{}", userId, couponId);
                        continue;
                    }
                    try {
                        this.obtainCoupou(request, userId, couponId);
                    } finally {
                        obtain.unlock();
                    }
                }
            }
        } catch (Exception e) {
            log.info("【优惠券发放失败】", e);
        }
        return couponObtainBeans;
    }

    /**
     * 领取优惠券
     *
     * @param userId
     * @throws RemoteServiceException
     */
    public void obtainCoupou(HttpServletRequest request, String userId, String couponId) throws RemoteServiceException, InterruptedException {
        Coupon coupon = couponService.getById(couponId);
        //限领张数
        Integer receiveLimitCount = coupon.getReceiveLimitCount();
        //剩余可领取数量
        Integer leftAmount = coupon.getLeftAmount();
        //已领取数量
        Integer receiveAmount = coupon.getReceiveAmount() == null ? 0 : coupon.getReceiveAmount();
        if (leftAmount <= 0) {
            throw new RemoteServiceException("领取优惠券失败,优惠券已被领取完");
        }
        Long count = userCouponService.lambdaQuery()
                .eq(UserCoupon::getUserId, userId)
                .eq(UserCoupon::getCouponId, couponId).count();
        if (receiveLimitCount != 0 && count >= receiveLimitCount) {
            throw new RemoteServiceException("领取失败,你已领取过券");
        }
        //实际领取张数
        int num = leftAmount - receiveLimitCount < 0 ? 0 : leftAmount - receiveLimitCount;
        //维护券的剩余数量和领取数量
        coupon.setLeftAmount(leftAmount - num);
        coupon.setReceiveAmount(receiveAmount + num);
        coupon.updateById();
        User user = userService.getById(userId);
        //Boolean transferType = user.getType().equals(UserTypeEnum.SERVICE.toString());
        //新增我的优惠券
        List<UserCoupon> list = new ArrayList<>();
        Date now = new Date();
        for (int i = 0; i < receiveLimitCount; i++) {
            UserCoupon userCoupon = new UserCoupon();
            userCoupon.setCouponId(couponId);
            userCoupon.setUserId(userId);
            userCoupon.setReceiveTime(now);
            userCoupon.setCouponName(coupon.getCouponName());
            userCoupon.setCouponType(coupon.getCouponType());
            userCoupon.setOrderAmount(coupon.getOrderAmount());
            userCoupon.setCouponValue(coupon.getCouponValue());
            userCoupon.setStatus(false);
            //userCoupon.setTransferType(transferType);
            if (coupon.getActiveDay() == null) {
                coupon.setActiveDay(0);
            }
            if (coupon.getActiveType().equals(CouponActiveTypeEnum.COMMON.getActiveType())) {
                userCoupon.setActiveStartTime(coupon.getActiveStartTime());
                userCoupon.setActiveEndTime(coupon.getActiveEndTime());
            }
            //当日起多少天
            if (coupon.getActiveType().equals(CouponActiveTypeEnum.TODAY.getActiveType())) {
                userCoupon.setActiveStartTime(DateUtil.beginOfDay(DateUtil.date()));
                userCoupon.setActiveEndTime(DateUtil.beginOfDay(DateUtil.offsetDay(DateUtil.date(), coupon.getActiveDay())));
            }
            //次日起多少天
            if (coupon.getActiveType().equals(CouponActiveTypeEnum.NEXTDAY.getActiveType())) {
                userCoupon.setActiveStartTime(DateUtil.beginOfDay(DateUtil.offsetDay(DateUtil.date(), 1)));
                userCoupon.setActiveEndTime(DateUtil.beginOfDay(DateUtil.offsetDay(DateUtil.date(), coupon.getActiveDay() + 1)));
            }
            userCoupon.setState(now.before(coupon.getActiveEndTime()) && now.after(coupon.getActiveStartTime()));
            userCoupon.setCompanyWechatId(user.getCompanyWechatId());
            userCoupon.setCompanyName(user.getCompanyName());
            list.add(userCoupon);
        }
        userCouponService.saveBatch(list);
    }

    public List<CustomCoupouBean> useableCoupou(String userId, BigDecimal orderAmount, List<String> goodsSpecIds) {

        List<CustomCoupouBean> customCoupouBeans = customCoupouMapper.listCoupou(userId, orderAmount, goodsSpecIds);
        return customCoupouBeans;
    }

    /**
     * 券详情
     *
     * @return
     */
    public UserCoupon miniDetail(String userCouponId) {

        return userCouponService.getById(userCouponId);
    }


    public CouponQrcodeBean qrcode(String userCouponId) throws RemoteServiceException {

        UserCoupon userCoupon = userCouponService.lambdaQuery()
                .eq(UserCoupon::getId, userCouponId)
                .eq(UserCoupon::getStatus, false)
                .eq(UserCoupon::getTransferType, true)
                .ge(UserCoupon::getActiveEndTime, new Date())
                .one();
        if (userCoupon == null) {
            throw new RemoteServiceException("优惠券不可转赠");
        }
        String qrcode = userCoupon.getTransferQrcode();
        if (StringUtils.isEmpty(qrcode)) {
            qrcode = wechatLogic.getQrcode(QrCodeEnum.COUPON.toString().toLowerCase(Locale.ROOT), userCoupon.getId(), userCoupon.getUserId(), userCoupon.getCompanyWechatId());
            if (StringUtils.isNotEmpty(qrcode)) {
                userCoupon.setTransferQrcode(qrcode);
                userCouponService.updateById(userCoupon);
            }
        }
        Coupon coupon = couponService.getById(userCoupon.getCouponId());
        CouponQrcodeBean couponQrcodeBean = new CouponQrcodeBean();
        couponQrcodeBean.setQrcode(qrcode);
        couponQrcodeBean.setLeftShareTimes(userCoupon.getLeftShareTimes());
        couponQrcodeBean.setShareTimes(coupon.getShareTimes());
        return couponQrcodeBean;
    }

    @Transactional
    public String transferCoupon(HttpServletRequest request, String userCouponId, UserLogic userLogic) throws RemoteServiceException {
        UserCoupon userCoupon = userCouponService.lambdaQuery()
                .eq(UserCoupon::getId, userCouponId)
                .eq(UserCoupon::getStatus, false)
                .eq(UserCoupon::getTransferType, true)
                .ge(UserCoupon::getActiveEndTime, new Date())
                .one();
        if (userCoupon == null || userCoupon.getLeftShareTimes() < 1) {
            throw new RemoteServiceException("优惠券已被领完");
        }
        String userId = CommonUtils.getUserId(request);
        if (userId.equals(userCoupon.getUserId())) {
            throw new RemoteServiceException("不能转赠自己");
        }
        User user = userService.lambdaQuery()
                .eq(User::getUserId, userId)
                .eq(User::getStatus, true)
                .one();
        if (user == null) {
            throw new RemoteServiceException("用户不存在");
        }
        if (user.getType().equals(UserTypeEnum.SERVICE.toString())) {
            throw new RemoteServiceException("业务员不能领取");
        }

        Long count = userCouponService.lambdaQuery()
                .eq(UserCoupon::getUserId, userId)
                .eq(UserCoupon::getCouponId, userCoupon.getCouponId())
                .count();

        if (count > 0) {
            throw new RemoteServiceException("领取失败,领取优惠券超过限制数量");
        }


        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.LOCK_COUPON_TRANSFER + userCoupon.getId());
        String transferCouponId;
        try {
            if (!obtain.tryLock(10, TimeUnit.SECONDS)) {
                throw new RemoteServiceException("系统繁忙，请稍后再试");
            }
            UserCoupon reObtainCoupon = this.miniDetail(userCouponId);
            if (reObtainCoupon.getLeftShareTimes() < 1) {
                throw new RemoteServiceException("优惠已被用户使用");
            }

            transferCouponId = this.transfer(user.getUserId(), userCouponId, user.getCompanyWechatId(), user.getCompanyName(), userLogic);
        } catch (InterruptedException e) {
            log.error("", e);
            throw new RemoteServiceException("系统繁忙，请稍后再试");
        } finally {
            obtain.unlock();
        }
        return transferCouponId;
    }

    protected String transfer(String userId, String userCouponId, String companyWechatId, String companyName, UserLogic userLogic) {
        UserCoupon userCoupon = userCouponService.lambdaQuery()
                .eq(UserCoupon::getId, userCouponId)
                .eq(UserCoupon::getStatus, false)
                .eq(UserCoupon::getTransferType, true)
                .ge(UserCoupon::getActiveEndTime, new Date())
                .one();
        String transferCouponId = null;
        if (userCoupon != null && userCoupon.getLeftShareTimes() > 0) {
            userLogic.bind(userId, userCoupon.getUserId());
            coustomUserCouponMapper.decreaseLeftTimes(userCoupon.getId());
            UserCoupon transferUserCoupon = new UserCoupon();
            BeanUtils.copyProperties(userCoupon, transferUserCoupon, "id");
            transferUserCoupon.setId(null);
            transferUserCoupon.setTransferType(false);
            transferUserCoupon.setReceiveTime(new Date());
            transferUserCoupon.setTransferQrcode(null);
            transferUserCoupon.setTransferor(userCoupon.getId());
            transferUserCoupon.setLeftShareTimes(0);
            transferUserCoupon.setUserId(userId);
            transferUserCoupon.setCompanyWechatId(companyWechatId);
            transferUserCoupon.setCompanyName(companyName);
            userCouponService.save(transferUserCoupon);
            transferCouponId = transferUserCoupon.getId();

            userCoupon.setLeftShareTimes(userCoupon.getLeftShareTimes() - 1);
            userCoupon.setStatus(userCoupon.getLeftShareTimes() == 0);
            userCoupon.updateById();
        }
        return transferCouponId;
    }

    /**
     * 检查优惠券是否可用
     */
    public UserCouponBean check(String userCouponId, BigDecimal payment, List<String> goodsIds, Boolean use) throws
            RemoteServiceException, InterruptedException {

        if (StringUtils.isEmpty(userCouponId)) {
            return null;
        }
        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.LOCK_COUPON_CHECK + userCouponId);
        try {
            if (!obtain.tryLock(5, TimeUnit.SECONDS)) {
                throw new RemoteServiceException("系统繁忙，请稍后再试");
            }
            //使用了优惠券
            if (StringUtils.isNotEmpty(userCouponId)) {
                UserCoupon userCoupon = this.miniDetail(userCouponId);

                UserCouponBean userCouponBean = new UserCouponBean();

                if (!userCoupon.getState()) {
                    throw new RemoteServiceException("下单失败,优惠券不可用");
                }
                if (userCoupon.getStatus()) {
                    throw new RemoteServiceException("优惠券已使用");
                }
                if (userCoupon.getTransferType() && userCoupon.getLeftShareTimes() < 1) {
                    throw new RemoteServiceException("优惠已被用户使用");
                }


                long curTime = new Date().getTime();
                if (userCoupon.getActiveStartTime().getTime() > curTime || userCoupon.getActiveEndTime().getTime() < curTime) {
                    throw new RemoteServiceException("优惠券使用时间范围不符合要求");
                }
                if (userCoupon.getCouponType().equals(CouponTypeEnum.SATISFY.toString())) {
                    log.info("【试用优惠券条件】券试用条件:{},订单金额：{}", userCoupon.getOrderAmount().doubleValue(), payment.doubleValue());
                    if (userCoupon.getOrderAmount().doubleValue() != 0 && userCoupon.getOrderAmount().doubleValue() > payment.doubleValue()) {
                        throw new RemoteServiceException("优惠券不符合使用要求");
                    }
                } else if (userCoupon.getCouponType().equals(CouponTypeEnum.GOODS.toString())) {

                    List<CouponGoods> couponGoodsList = couponGoodsService.lambdaQuery()
                            .eq(CouponGoods::getCouponId, userCoupon.getCouponId())
                            .in(CouponGoods::getGoodsId, goodsIds)
                            .list();
                    if (couponGoodsList.size() == 0) {
                        throw new RemoteServiceException("优惠券不符合使用要求");
                    }
                    userCouponBean.setGoodsId(couponGoodsList.get(0).getGoodsId());
                }

                if (userCoupon.getTransferType()) {
                    if (userCoupon.getLeftShareTimes() < 1) {
                        throw new RemoteServiceException("优惠已被用户使用");
                    }
                }
                //将券改为已使用状态
                UserCoupon userCouponRet = this.cancelCoupon(userCoupon.getId(), use);
                BeanUtils.copyProperties(userCouponRet, userCouponBean, "goodsId");

                return userCouponBean;
            }
        } finally {
            obtain.unlock();
        }
        return null;
    }

    /**
     * 核销优惠券,可转增的券全部是可使用状态
     *
     * @param userCouponId
     */
    @Transactional
    public UserCoupon cancelCoupon(String userCouponId, Boolean use) {
        UserCoupon userCoupon = userCouponService.getById(userCouponId);
        UserCoupon userCouponCopy = null;
        if (userCoupon != null && userCoupon.getTransferType() != null && userCoupon.getTransferType() && userCoupon.getLeftShareTimes() > 0) {
            //数量减一,增加一条新的记录
            if (use) {
                userCoupon.setLeftShareTimes(userCoupon.getLeftShareTimes() - 1);
                userCoupon.setStatus(userCoupon.getLeftShareTimes() == 0);
                userCoupon.updateById();
            }
        } else {
            userCouponService.lambdaUpdate()
                    .set(UserCoupon::getStatus, use)
                    .set(UserCoupon::getUsedTime, new Date())
                    .eq(UserCoupon::getId, userCouponId)
                    .update();
        }
        return userCouponCopy == null ? userCoupon : userCouponCopy;
    }

    /**
     * 订单取消,恢复优惠券
     *
     * @param userCouponIds 优惠券id数组
     */
    @Transactional
    public void updateStatus(List<String> userCouponIds) {
        userCouponService.lambdaUpdate()
                .set(UserCoupon::getStatus, false)
                .set(UserCoupon::getUsedTime, null)
                .in(UserCoupon::getId, userCouponIds)
                .update();
        for (String userCouponId : userCouponIds) {
            if (StringUtils.isEmpty(userCouponId)) {
                continue;
            }
            log.info("订单取消,恢复用户优惠券id{}", userCouponId);
            UserCoupon userCoupon = userCouponService.getById(userCouponId);
            if(userCoupon == null)
                continue;
            if(userCoupon.getTransferType() == null || !userCoupon.getTransferType())
                continue;

            coustomUserCouponMapper.increaseLeftTimes(userCouponId);
        }
    }
}
