package com.gree.mall.contest.logic.goods;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.bean.goods.GoodsFavoriteBean;
import com.gree.mall.contest.bean.user.CurrentCompanyWechat;
import com.gree.mall.contest.commonmapper.CustomGoodsMapper;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.plus.entity.GoodsFavorite;
import com.gree.mall.contest.plus.service.GoodsFavoriteService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;

@Service
@Slf4j
@RequiredArgsConstructor
public class GoodsFavoriteLogic {

    private final GoodsFavoriteService goodsFavoriteService;
    private final CustomGoodsMapper customGoodsMapper;
    private final CommonLogic commonLogic;

    public GoodsFavorite saveGoodsFavorite(HttpServletRequest request, String userId, String goodsId, String goodsSpecId) throws Exception {
        CurrentCompanyWechat companyWechat = commonLogic.getCurrentCompanyWechat(request);

        if (StringUtils.isBlank(userId)) {
            throw new RemoteServiceException("无效的userId");
        }
        if (StringUtils.isBlank(goodsId)) {
            throw new RemoteServiceException("无效的goodsId");
        }
        GoodsFavorite goodsFavorite = new GoodsFavorite();
        goodsFavorite.setGoodsId(goodsId);
        goodsFavorite.setUserId(userId);
        goodsFavorite.setGoodsSpecId(goodsSpecId);
        goodsFavorite.setCreateTime(new Date());
        goodsFavorite.setCompanyWechatId(companyWechat.getCurrentCompanyWechatId());
        goodsFavorite.setCompanyName(companyWechat.getCurrentCompanyName());
        goodsFavoriteService.save(goodsFavorite);
        return goodsFavorite;
    }

    public void delGoodsFavorite(String goodFavoriteId) {
        goodsFavoriteService.lambdaUpdate()
                .eq(StringUtils.isNotBlank(goodFavoriteId), GoodsFavorite::getGoodsFavoriteId, goodFavoriteId)
                .remove();
    }

    public void detailDelGoodsFavorite(String goodsId, String userId, String goodSpecId) throws RemoteServiceException {
        goodsFavoriteService.lambdaUpdate()
                .eq(StringUtils.isNotBlank(goodsId), GoodsFavorite::getGoodsId, goodsId)
                .eq(StringUtils.isNotBlank(userId), GoodsFavorite::getUserId, userId)
                .eq(StringUtils.isNotBlank(goodSpecId), GoodsFavorite::getGoodsSpecId, goodSpecId)
                .remove();
    }


    public IPage<GoodsFavoriteBean> pageGoodsFavorite(String userId, Integer pageNum, Integer pageSize) {
        return customGoodsMapper.queryGoodsFavorite(new Page<>(pageNum, pageSize), userId);
    }


}

