package com.gree.mall.contest.logic.notice;

import cn.hutool.core.collection.CollectionUtil;
import com.aliyuncs.utils.StringUtils;
import com.gree.mall.contest.bean.admin.AdminUserCom;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.plus.entity.MiniNotice;
import com.gree.mall.contest.plus.service.MiniNoticeService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;


@Service
@Slf4j
@RequiredArgsConstructor
public class MiniNoticeLogic {
    private final MiniNoticeService miniNoticeService;
    private final CommonLogic commonLogic;

    /**
     * 新增
     *
     * @param miniNotice
     */
    public void add(HttpServletRequest request, MiniNotice miniNotice) {
        //获取当前登录企业微信id
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        if (StringUtils.isEmpty(adminUser.getLoginCompanyWechatId())) {
            throw new RemoteServiceException("无效微信企业id");
        }


        Date createDate = new Date();
        miniNotice.setCreateTime(createDate);
        miniNotice.setCompanyWechatId(adminUser.getLoginCompanyWechatId());
        miniNotice.setCompanyName(adminUser.getLoginCompanyName());
        miniNoticeService.save(miniNotice);
    }

    /**
     * 修改
     *
     * @param miniNotice
     */
    public void update(MiniNotice miniNotice) {
        miniNoticeService.updateById(miniNotice);
    }


    /**
     * 查询
     *
     * @return
     */
    public List<MiniNotice> select(HttpServletRequest request) {
        //获取当前登录企业微信id
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        List<String> companyWechatIds = adminUser.getCompanyWechatIds();


        List<MiniNotice> list = miniNoticeService.lambdaQuery()
                .in(CollectionUtil.isNotEmpty(companyWechatIds), MiniNotice::getCompanyWechatId, companyWechatIds)
                .list();
        return list;
    }


}
