package com.gree.mall.contest.logic.store;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.gree.mall.contest.bean.admin.AdminUserCom;
import com.gree.mall.contest.bean.store.PlatformStoreVO;
import com.gree.mall.contest.bean.zfire.ZfireParamBean;
import com.gree.mall.contest.commonmapper.CommonMapper;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.plus.entity.PlatformStore;
import com.gree.mall.contest.plus.service.PlatformStoreService;
import com.gree.mall.contest.utils.zfire.FieldUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

@Service
@Slf4j
@RequiredArgsConstructor
public class PlatformStoreLogic {

    private final CommonMapper commonMapper;
    private final CommonLogic commonLogic;
    private final PlatformStoreService platformStoreService;

    public IPage<PlatformStoreVO> list(ZfireParamBean zfireParamBean) {
        AdminUserCom adminUser = commonLogic.getAdminUser();
        zfireParamBean = FieldUtils.supplyParam(zfireParamBean, PlatformStoreVO.class);
        IPage<PlatformStoreVO> page = commonMapper.platformStorePage(zfireParamBean.page(), zfireParamBean);
        return page;
    }

    public PlatformStore detail(String id) {
        final PlatformStore platformStore = platformStoreService.getById(id);
        return platformStore;
    }

    public void add(PlatformStore platformStore) {
        checkParams(platformStore);

        //判断是否存在
        PlatformStore old = platformStoreService.getById(platformStore.getStoreCode());
        if (old != null) {
            throw new RemoteServiceException("编号【" + old.getId() + "】已存在");
        }

        platformStore.insert();
    }

    public void update(PlatformStore platformStore) {
        checkParams(platformStore);

        //编号不能改
        platformStore.setStoreCode(null);
        //门店所属平台不能改
        platformStore.setType(null);
        platformStore.updateById();

    }

    private void checkParams(PlatformStore platformStore) {
        if (StringUtils.isBlank(platformStore.getStoreCode())) {
            throw new RemoteServiceException("商家编号必填");
        }
        if (StringUtils.isBlank(platformStore.getStoreName())) {
            throw new RemoteServiceException("商家名称必填");
        }
        if (StringUtils.isBlank(platformStore.getType())) {
            throw new RemoteServiceException("门店所属平台必选");
        }
    }
}
